@@ -14,7 +14,7 @@ our @EXPORT_OK = ( qw/ get_gsl_version get_step_types / );
 our %EXPORT_TAGS;
 push @{$EXPORT_TAGS{all}}, @EXPORT, @EXPORT_OK;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 $VERSION = eval $VERSION;
 
 our $Verbose = 0;
@@ -23,17 +23,28 @@ require XSLoader;
 XSLoader::load('Math::GSLx::ODEIV2', $VERSION);
 
 my %step_types = (
-  rk2   => 1,
-  rk4   => 2,
-  rkf45 => 3,
-  rkck  => 4,
-  rk8pd => 5,
+  rk2   	=> 1,
+  rk4   	=> 2,
+  rkf45 	=> 3,
+  rkck  	=> 4,
+  rk8pd 	=> 5,
+  rk1imp_j	=> 6,
+  rk2imp_j	=> 7,
+  rk4imp_j	=> 8,  
+  bsimp_j 	=> 9,
+  msadams	=> 10,
+  msbdf_j	=> 11,
 );
 
 sub ode_solver {
 
   my ($eqn, $t_range, $opts) = @_;
-  croak "First argument must be a code reference" unless (ref $eqn eq 'CODE');
+  my $jac;
+  if (ref $eqn eq 'ARRAY') {
+    $jac = $eqn->[1] if defined $eqn->[1];
+    $eqn = $eqn->[0];
+  }
+  croak "First argument must specify one or more code references" unless (ref $eqn eq 'CODE');
 
   ## Parse Options ##
 
@@ -59,8 +70,24 @@ sub ode_solver {
     $step_type = $step_types{rk8pd};
   }
 
-  # Initial h_step
-  my $h_step = (exists $opts->{h_step}) ? $opts->{h_step} : 1e-6;
+  # h step configuration
+  my $h_init;
+  my $h_max  = (exists $opts->{h_max} ) ? $opts->{h_max}  : 0;
+  if (exists $opts->{h_init}) {
+    $h_init = $opts->{h_init};
+
+    # if the user specifies an h_init greater than h_max then croak
+    if ($h_max && ($h_init > $h_max)) {
+      croak "h_init cannot be set greater than h_max";
+    }
+  } else {
+    $h_init = 1e-6;
+
+    # if the default h_init would be greater tha h_max then set h_init = h_max
+    if ($h_max && ($h_init > $h_max)) {
+      $h_init = $h_max;
+    }
+  }
 
   # Error levels
   my $epsabs = (exists $opts->{epsabs}) ? $opts->{epsabs} : 1e-6;
@@ -89,7 +116,7 @@ sub ode_solver {
   {
     local @_; #be sure the stack is clear before calling c_ode_solver!
     $result = c_ode_solver(
-      $eqn, @t_range, $step_type, $h_step, $epsabs, $epsrel, $a_y, $a_dydt
+      $eqn, $jac, @t_range, $step_type, $h_init, $h_max, $epsabs, $epsrel, $a_y, $a_dydt
     );
   }
 
@@ -144,17 +171,29 @@ Why the C<Math::GSLx::> namespace? Well since Jonathan Leto has been kind enough
 
 This module is in an alpha state. It needs more tests and the ability to configure more of the options that the GSL library allows. Currently this module leans on the fact that GSL has an extensive test suite. While the author has put some thought into the interface it may change in the future as the above mentioned functionality is added or as bugs appear. Bug reports are encouraged!
 
+Also, as of version 0.06, support for including a Jacobian of the system has been added, including the step types that this allows, however this functionality is almost totally untested. Until some of the stiff/extreme test cases can be ported from GSL the author is not certain the the functionality has been properly implemented. Sadly C<t/sine.*> pass even when not properly implemented, which is unnerving. I<Caveat emptor>.
+
 =head1 EXPORTED FUNCTIONS
 
 =head2 ode_solver
 
 This is the main function of the module. 
 
- $solution = ode_solver( $diffeq_code_ref, $t_range [, $opts_hashref ])
+ $solution = ode_solver( $diffeq_code_ref, $t_range)
 
-=head3 required arguments
+or
 
-The first argument, C<$diffeq_code_ref>, is a code reference to a subroutine (or anonymous sub) which specifies the differential equations. This subroutine must have a specific construction:
+ $solution = ode_solver( $diffeq_code_ref, $t_range, $opts_hashref)
+
+or
+
+ $solution = ode_solver( [$diffeq_code_ref, $jacobian_code_ref], $t_range, $opts_hashref)
+
+Before detailing how to call C<ode_solver>, lets see how to construct the differential equation system.
+
+=head3 the differential equation system
+
+The differential equation system is defined in a code reference (in the example C<$diffeq_code_ref>). This code reference (or anonymous subroutine) must have a specific construction:
 
 =over 
 
@@ -166,12 +205,49 @@ If called without arguments (i.e. C<< $diffeq_code_ref->() >>) it should return
 
 When called with arguments, the first argument will be time (or the independent parameter) and the rest will be the function values in the same order as the initial conditions. The returns in this case should be the values of the derivatives of the function values. 
 
+If one or more of the returned values are not numbers (as determined by L<Scalar::Util> C<looks_like_number>), the solver will immediately return all calculations up until (and not including) this step, accompanied by a warning. This may be done intentionally to exit the solve routine earlier than the end time specified in the second argument.
+
 =item *
 
 Please note that as with other differential equation solvers, any higher order differential equations must be converted into systems of first order differential equations. 
 
 =back
 
+Optionally the system may be further described with a code reference which defines the Jacobian of the system (in the example C<$jacobian_code_ref>). Again, this code reference has a specific construction. The arguments will be passed in exactly the same way as for the equations code reference (though it will not be called without arguments). The returns should be two array references. 
+
+=over
+
+=item *
+
+The first is the Jacobian matrix formed as an array reference containing array references. It should be square where each dimension is equal to the number of differential equations. Each "row" contains the derivatives of the related differential equations with respect to each dependant parameter, respectively.
+
+ [
+  [ d(dy[0]/dt)/d(y[0]), d(dy[0]/dt)/d(y[1]), ... ],
+  [ d(dy[1]/dt)/d(y[0]), d(dy[1]/dt)/d(y[1]), ... ],
+  ...
+  [ ..., d(dy[n]/dt)/d(y[n])],
+ ]
+
+=item *
+
+The second returned array reference contains the derivatives of the differential equations with respect to the independant parameter.
+
+ [ d(dy[0]/dt)/dt, ..., d(dy[n]/dt)/dt ]
+
+=back
+
+The Jacobian code reference is only needed for certain step types, those step types whose names end in C<_j>.
+
+=head3 required arguments
+
+C<ode_solver> requires two arguments, they are as follows:
+
+=head4 first argument
+
+The first argument may be either a code reference or an array reference containing one or two code references. In the single code reference form this represents the differential equation system, constructed as described above. In the array reference form, the first argument must be the differential equation system code reference, but now optionally a code reference for the Jacobian of the system may be supplied as the second item.
+
+=head4 second argument
+
 The second argument, C<$t_range>, specifies the time values that are used for the calculation. This may be used one of two ways:
 
 =over
@@ -194,11 +270,15 @@ The third argument, C<$opts_hashref>, is a hash reference containing other optio
 
 =item *
 
-C<type> specifies the step type to be used. The default is C<rk8pd>. The available step types can be found using the exportable function L</get_step_types>. They are those steps defined by the C<gsl_odeiv2> library which do not need special extras, most commonly this means those that do not require the Jacobian of the system.
+C<type> specifies the step type to be used. The default is C<rk8pd>. The available step types can be found using the exportable function L</get_step_types>. Those step types whose name ends in C<_j> require the Jacobian.
 
 =item *
 
-C<h_step> the initial "h" step used by the solver. Defaults to C<1e-6>.
+C<h_init> the initial "h" step used by the solver. Defaults to C<1e-6>.
+
+=item *
+
+C<h_max> the maximum "h" step allowed to the adaptive step size solver. Set to zero to use the default value specified the GSL, this is the default behavior if unspecified. Note: the module will croak if C<h_init> is set greater than C<h_max>, however if C<h_init> is not specified and the default would violate this relation, C<h_init> will be set to C<h_max> implicitly.
 
 =item * Error scaling options. These all refer to the adaptive step size contoller which is well documented in the L<GSL manual|http://www.gnu.org/software/gsl/manual/html_node/Adaptive-Step_002dsize-Control.html>. 
 
@@ -238,7 +318,7 @@ For a usage example see the L</SYNOPSIS> for a sine function given by C<y''(t)=-
 
 =head2 get_step_types
 
-Returns the available step types which may be specified in the L</ode_solver> function's options hashref.
+Returns the available step types which may be specified in the L</ode_solver> function's options hashref. Note that those step types whose name end in C<_j> require the Jacobian.
 
 =head2 get_gsl_version
 
